#include <stdio.h>
#include <stdlib.h>
#include "glFile.h"

modelData * loadModel( const char *fileName )
{
   modelData *model = NULL;
   FILE *hFile = fopen( fileName, "rb" );

   /* check the file open command */
   if( hFile != NULL )
   {
      int fileLen;
      char *buffer;
      modelHeader *mdh;
      frame *frm;
      coord *pointList;
      triangleIndex *triIndex;
      mesh *bufIndexPtr;
      int i;

      /* find length of file */
      fseek( hFile, 0, SEEK_END );
      fileLen = ftell( hFile );
      fseek( hFile, 0, SEEK_SET );

      /* read in file */
      buffer = malloc( fileLen+1 );
      fread( buffer, sizeof( char ), fileLen, hFile );

      /* get pointer to file header */
      mdh = (modelHeader *)buffer;

      /* construct the model data structure */
      model = (modelData *)malloc( sizeof( modelData ));

      /* create a pointlist */
      pointList = (coord *)malloc( sizeof( coord ) * mdh->numXYZ );
      model->pointList = pointList;
      model->numPoints = mdh->numXYZ;

      /* offset to points in first frame */
      frm = (frame *)&buffer[mdh->offsetFrames];

      for( i=0; i<mdh->numXYZ; i++ )
      {
         pointList[i].pt[0] = frm->scale[0] * frm->fp[i].v[0] + frm->translate[0];
         pointList[i].pt[1] = frm->scale[1] * frm->fp[i].v[1] + frm->translate[1];
         pointList[i].pt[2] = frm->scale[2] * frm->fp[i].v[2] + frm->translate[2];
      }

      /* create a mesh list */
      triIndex = (triangleIndex *)malloc( sizeof( coord ) * mdh->numTris );
      model->numTriangles = mdh->numTris;
      model->triIndex = triIndex;

      /* point to indexes in buffers */
      bufIndexPtr = (mesh *)&buffer[mdh->offsetTris];

      for( i=0; i<mdh->numTris; i++ )
      {
         triIndex[i].index[0] = bufIndexPtr[i].meshIndex[0];
         triIndex[i].index[1] = bufIndexPtr[i].meshIndex[1];
         triIndex[i].index[2] = bufIndexPtr[i].meshIndex[2];
      }

      /* cleanup */
      free( buffer );
   }
   return( model );
}

